@@ -70,6 +70,7 @@
 #define PKG_INTERNAL_CFLAGS		(((uint64_t) 1) << 42)
 #define PKG_DUMP_PERSONALITY		(((uint64_t) 1) << 43)
 #define PKG_SHARED			(((uint64_t) 1) << 44)
+#define PKG_DUMP_LICENSE		(((uint64_t) 1) << 45)
 
 static pkgconf_client_t pkg_client;
 static const pkgconf_fragment_render_ops_t *want_render_ops = NULL;
@@ -85,7 +86,7 @@ FILE *error_msgout = NULL;
 FILE *logfile_out = NULL;
 
 static bool
-error_handler(const char *msg, const pkgconf_client_t *client, const void *data)
+error_handler(const char *msg, const pkgconf_client_t *client, void *data)
 {
 	(void) client;
 	(void) data;
@@ -346,16 +347,6 @@ apply_path(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int max
 	return true;
 }
 
-static void
-print_variable(pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *variable)
-{
-	const char *var;
-
-	var = pkgconf_tuple_find(client, &pkg->vars, variable);
-	if (var != NULL)
-		printf("%s", var);
-}
-
 static bool
 apply_variable(pkgconf_client_t *client, pkgconf_pkg_t *world, void *variable, int maxdepth)
 {
@@ -366,11 +357,12 @@ apply_variable(pkgconf_client_t *client, pkgconf_pkg_t *world, void *variable, i
 	{
 		pkgconf_dependency_t *dep = iter->data;
 		pkgconf_pkg_t *pkg = dep->match;
+		const char *var;
 
-		if (iter->prev != NULL)
-			printf(" ");
+		var = pkgconf_tuple_find(client, &pkg->vars, variable);
 
-		print_variable(client, pkg, variable);
+		if (var != NULL)
+			printf("%s%s", iter->prev != NULL ? " " : "", var);
 	}
 
 	printf("\n");
@@ -593,6 +585,32 @@ apply_simulate(pkgconf_client_t *client, pkgconf_pkg_t *world, void *data, int m
 }
 #endif
 
+static void
+print_license(pkgconf_client_t *client, pkgconf_pkg_t *pkg, void *data)
+{
+	(void) client;
+	(void) data;
+
+	if (pkg->flags & PKGCONF_PKG_PROPF_VIRTUAL)
+		return;
+
+	/* NOASSERTION is the default when the license is unknown, per SPDX spec § 3.15 */
+	printf("%s: %s\n", pkg->id, pkg->license != NULL ? pkg->license : "NOASSERTION");
+}
+
+static bool
+apply_license(pkgconf_client_t *client, pkgconf_pkg_t *world, void *data, int maxdepth)
+{
+	int eflag;
+
+	eflag = pkgconf_pkg_traverse(client, world, print_license, data, maxdepth, 0);
+
+	if (eflag != PKGCONF_PKG_ERRF_OK)
+		return false;
+
+	return true;
+}
+
 static void
 version(void)
 {
@@ -696,6 +714,7 @@ usage(void)
 	printf("  --path                            show the exact filenames for any matching .pc files\n");
 	printf("  --modversion                      print the specified module's version to stdout\n");
 	printf("  --internal-cflags                 do not filter 'internal' cflags from output\n");
+	printf("  --license                         print the specified module's license to stdout if known\n");
 
 	printf("\nfiltering output:\n\n");
 #ifndef PKGCONF_LITE
@@ -801,8 +820,13 @@ main(int argc, char *argv[])
 	char *logfile_arg = NULL;
 	char *want_env_prefix = NULL;
 	unsigned int want_client_flags = PKGCONF_PKG_PKGF_NONE;
-	pkgconf_cross_personality_t *personality;
+	pkgconf_cross_personality_t *personality = NULL;
 	bool opened_error_msgout = false;
+	pkgconf_pkg_t world = {
+		.id = "virtual:world",
+		.realname = "virtual world package",
+		.flags = PKGCONF_PKG_PROPF_STATIC | PKGCONF_PKG_PROPF_VIRTUAL,
+	};
 
 	want_flags = 0;
 
@@ -883,6 +907,7 @@ main(int argc, char *argv[])
 		{ "dump-personality", no_argument, &want_flags, PKG_DUMP_PERSONALITY },
 		{ "personality", required_argument, NULL, 53 },
 #endif
+		{ "license", no_argument, &want_flags, PKG_DUMP_LICENSE },
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -894,12 +919,6 @@ main(int argc, char *argv[])
 	}
 #endif
 
-#ifndef PKGCONF_LITE
-	personality = deduce_personality(argv);
-#else
-	personality = pkgconf_cross_personality_default();
-#endif
-
 	while ((ret = pkg_getopt_long_only(argc, argv, "", options, NULL)) != -1)
 	{
 		switch (ret)
@@ -950,13 +969,21 @@ main(int argc, char *argv[])
 #endif
 		case '?':
 		case ':':
-			return EXIT_FAILURE;
-			break;
+			ret = EXIT_FAILURE;
+			goto out;
 		default:
 			break;
 		}
 	}
 
+	if (personality == NULL) {
+#ifndef PKGCONF_LITE
+		personality = deduce_personality(argv);
+#else
+		personality = pkgconf_cross_personality_default();
+#endif
+	}
+
 	pkgconf_path_copy_list(&personality->dir_list, &dir_list);
 	pkgconf_path_free(&dir_list);
 
@@ -1001,24 +1028,33 @@ main(int argc, char *argv[])
 	if ((want_flags & PKG_ABOUT) == PKG_ABOUT)
 	{
 		about();
-		return EXIT_SUCCESS;
+
+		ret = EXIT_SUCCESS;
+		goto out;
 	}
 
 	if ((want_flags & PKG_VERSION) == PKG_VERSION)
 	{
 		version();
-		return EXIT_SUCCESS;
+
+		ret = EXIT_SUCCESS;
+		goto out;
 	}
 
 	if ((want_flags & PKG_HELP) == PKG_HELP)
 	{
 		usage();
-		return EXIT_SUCCESS;
+
+		ret = EXIT_SUCCESS;
+		goto out;
 	}
 
 	if (getenv("PKG_CONFIG_FDO_SYSROOT_RULES"))
 		want_client_flags |= PKGCONF_PKG_PKGF_FDO_SYSROOT_RULES;
 
+	if (getenv("PKG_CONFIG_PKGCONF1_SYSROOT_RULES"))
+		want_client_flags |= PKGCONF_PKG_PKGF_PKGCONF1_SYSROOT_RULES;
+
 	if ((want_flags & PKG_SHORT_ERRORS) == PKG_SHORT_ERRORS)
 		want_client_flags |= PKGCONF_PKG_PKGF_SIMPLIFY_ERRORS;
 
@@ -1074,17 +1110,11 @@ main(int argc, char *argv[])
 	if ((want_flags & PKG_INTERNAL_CFLAGS) == PKG_INTERNAL_CFLAGS)
 		want_client_flags |= PKGCONF_PKG_PKGF_DONT_FILTER_INTERNAL_CFLAGS;
 
-#ifdef XXX_NOTYET
 	/* if these selectors are used, it means that we are inquiring about a single package.
 	 * so signal to libpkgconf that we do not want to use the dependency resolver for more than one level,
 	 * and also limit the SAT problem to a single package.
-	 *
-	 * i disabled this because too many upstream maintainers are still invoking pkg-config correctly to have
-	 * the more sane behaviour as default.  use --maximum-traverse-depth=1 or PKG_CONFIG_MAXIMUM_TRAVERSE_DEPTH
-	 * environment variable to get the same results in meantime.
 	 */
-	if ((want_flags & PKG_EXISTS) == 0 &&
-		((want_flags & PKG_REQUIRES) == PKG_REQUIRES ||
+	if (((want_flags & PKG_REQUIRES) == PKG_REQUIRES ||
 		(want_flags & PKG_REQUIRES_PRIVATE) == PKG_REQUIRES_PRIVATE ||
 		(want_flags & PKG_PROVIDES) == PKG_PROVIDES ||
 		(want_flags & PKG_VARIABLES) == PKG_VARIABLES ||
@@ -1095,7 +1125,6 @@ main(int argc, char *argv[])
 		maximum_package_count = 1;
 		maximum_traverse_depth = 1;
 	}
-#endif
 
 	if (getenv("PKG_CONFIG_ALLOW_SYSTEM_CFLAGS") != NULL)
 		want_flags |= PKG_KEEP_SYSTEM_CFLAGS;
@@ -1106,6 +1135,12 @@ main(int argc, char *argv[])
 	if ((builddir = getenv("PKG_CONFIG_TOP_BUILD_DIR")) != NULL)
 		pkgconf_client_set_buildroot_dir(&pkg_client, builddir);
 
+	if ((want_flags & PKG_REQUIRES_PRIVATE) == PKG_REQUIRES_PRIVATE ||
+		(want_flags & PKG_CFLAGS))
+	{
+		want_client_flags |= PKGCONF_PKG_PKGF_SEARCH_PRIVATE;
+	}
+
 	if ((sysroot_dir = getenv("PKG_CONFIG_SYSROOT_DIR")) != NULL)
 	{
 		const char *destdir;
@@ -1128,21 +1163,25 @@ main(int argc, char *argv[])
 	if (required_pkgconfig_version != NULL)
 	{
 		if (pkgconf_compare_version(PACKAGE_VERSION, required_pkgconfig_version) >= 0)
-			return EXIT_SUCCESS;
+			ret = EXIT_SUCCESS;
+		else
+			ret = EXIT_FAILURE;
 
-		return EXIT_FAILURE;
+		goto out;
 	}
 
 	if ((want_flags & PKG_LIST) == PKG_LIST)
 	{
 		pkgconf_scan_all(&pkg_client, NULL, print_list_entry);
-		return EXIT_SUCCESS;
+		ret = EXIT_SUCCESS;
+		goto out;
 	}
 
 	if ((want_flags & PKG_LIST_PACKAGE_NAMES) == PKG_LIST_PACKAGE_NAMES)
 	{
 		pkgconf_scan_all(&pkg_client, NULL, print_package_entry);
-		return EXIT_SUCCESS;
+		ret = EXIT_SUCCESS;
+		goto out;
 	}
 
 	if (logfile_arg == NULL)
@@ -1156,7 +1195,7 @@ main(int argc, char *argv[])
 
 	if (required_module_version != NULL)
 	{
-		pkgconf_pkg_t *pkg;
+		pkgconf_pkg_t *pkg = NULL;
 		pkgconf_node_t *node;
 		pkgconf_list_t deplist = PKGCONF_LIST_INITIALIZER;
 
@@ -1175,18 +1214,28 @@ main(int argc, char *argv[])
 			{
 				if (want_flags & PKG_PRINT_ERRORS)
 					pkgconf_error(&pkg_client, "Package '%s' was not found\n", pkgiter->package);
-				return EXIT_FAILURE;
+
+				ret = EXIT_FAILURE;
+				goto cleanup;
 			}
 
 			if (pkgconf_compare_version(pkg->version, required_module_version) >= 0)
-				return EXIT_SUCCESS;
+			{
+				ret = EXIT_SUCCESS;
+				goto cleanup;
+			}
 		}
 
-		return EXIT_FAILURE;
+		ret = EXIT_FAILURE;
+cleanup:
+		if (pkg != NULL)
+			pkgconf_pkg_unref(&pkg_client, pkg);
+		pkgconf_dependency_free(&deplist);
+		goto out;
 	}
 	else if (required_exact_module_version != NULL)
 	{
-		pkgconf_pkg_t *pkg;
+		pkgconf_pkg_t *pkg = NULL;
 		pkgconf_node_t *node;
 		pkgconf_list_t deplist = PKGCONF_LIST_INITIALIZER;
 
@@ -1205,18 +1254,28 @@ main(int argc, char *argv[])
 			{
 				if (want_flags & PKG_PRINT_ERRORS)
 					pkgconf_error(&pkg_client, "Package '%s' was not found\n", pkgiter->package);
-				return EXIT_FAILURE;
+
+				ret = EXIT_FAILURE;
+				goto cleanup2;
 			}
 
 			if (pkgconf_compare_version(pkg->version, required_exact_module_version) == 0)
-				return EXIT_SUCCESS;
+			{
+				ret = EXIT_SUCCESS;
+				goto cleanup2;
+			}
 		}
 
-		return EXIT_FAILURE;
+		ret = EXIT_FAILURE;
+cleanup2:
+		if (pkg != NULL)
+			pkgconf_pkg_unref(&pkg_client, pkg);
+		pkgconf_dependency_free(&deplist);
+		goto out;
 	}
 	else if (required_max_module_version != NULL)
 	{
-		pkgconf_pkg_t *pkg;
+		pkgconf_pkg_t *pkg = NULL;
 		pkgconf_node_t *node;
 		pkgconf_list_t deplist = PKGCONF_LIST_INITIALIZER;
 
@@ -1235,14 +1294,24 @@ main(int argc, char *argv[])
 			{
 				if (want_flags & PKG_PRINT_ERRORS)
 					pkgconf_error(&pkg_client, "Package '%s' was not found\n", pkgiter->package);
-				return EXIT_FAILURE;
+
+				ret = EXIT_FAILURE;
+				goto cleanup3;
 			}
 
 			if (pkgconf_compare_version(pkg->version, required_max_module_version) <= 0)
-				return EXIT_SUCCESS;
+			{
+				ret = EXIT_SUCCESS;
+				goto cleanup3;
+			}
 		}
 
-		return EXIT_FAILURE;
+		ret = EXIT_FAILURE;
+cleanup3:
+		if (pkg != NULL)
+			pkgconf_pkg_unref(&pkg_client, pkg);
+		pkgconf_dependency_free(&deplist);
+		goto out;
 	}
 
 	while (1)
@@ -1286,85 +1355,68 @@ main(int argc, char *argv[])
 	if (pkgq.head == NULL)
 	{
 		fprintf(stderr, "Please specify at least one package name on the command line.\n");
-		return EXIT_FAILURE;
+		ret = EXIT_FAILURE;
+		goto out;
 	}
 
 	ret = EXIT_SUCCESS;
 
+	if (!pkgconf_queue_solve(&pkg_client, &pkgq, &world, maximum_traverse_depth))
+	{
+		ret = EXIT_FAILURE;
+		goto out;
+	}
+
 #ifndef PKGCONF_LITE
 	if ((want_flags & PKG_SIMULATE) == PKG_SIMULATE)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
 		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SKIP_ERRORS);
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_simulate, -1, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_simulate(&pkg_client, &world, NULL, -1);
 	}
 #endif
 
-	if (!pkgconf_queue_validate(&pkg_client, &pkgq, maximum_traverse_depth))
+	if ((want_flags & PKG_VALIDATE) == PKG_VALIDATE)
+		goto out;
+
+	if ((want_flags & PKG_DUMP_LICENSE) == PKG_DUMP_LICENSE)
 	{
-		ret = EXIT_FAILURE;
+		apply_license(&pkg_client, &world, &ret, 2);
 		goto out;
 	}
 
-	if ((want_flags & PKG_VALIDATE) == PKG_VALIDATE)
-		return 0;
-
 	if ((want_flags & PKG_UNINSTALLED) == PKG_UNINSTALLED)
 	{
 		ret = EXIT_FAILURE;
-		pkgconf_queue_apply(&pkg_client, &pkgq, apply_uninstalled, maximum_traverse_depth, &ret);
+		apply_uninstalled(&pkg_client, &world, &ret, 2);
 		goto out;
 	}
 
 	if (want_env_prefix != NULL)
 	{
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_env, maximum_traverse_depth, want_env_prefix))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
-
+		apply_env(&pkg_client, &world, want_env_prefix, 2);
 		want_flags = 0;
 	}
 
 	if ((want_flags & PKG_PROVIDES) == PKG_PROVIDES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_provides, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_provides(&pkg_client, &world, NULL, 2);
 	}
 
 #ifndef PKGCONF_LITE
 	if ((want_flags & PKG_DIGRAPH) == PKG_DIGRAPH)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_digraph, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_digraph(&pkg_client, &world, NULL, 2);
 	}
 #endif
 
 	if ((want_flags & PKG_MODVERSION) == PKG_MODVERSION)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_modversion, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_modversion(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_PATH) == PKG_PATH)
@@ -1372,22 +1424,13 @@ main(int argc, char *argv[])
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
 		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SKIP_ROOT_VIRTUAL);
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_path, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_path(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_VARIABLES) == PKG_VARIABLES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_variables, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_variables(&pkg_client, &world, NULL, 2);
 	}
 
 	if (want_variable)
@@ -1395,68 +1438,56 @@ main(int argc, char *argv[])
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
 		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SKIP_ROOT_VIRTUAL);
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_variable, maximum_traverse_depth, want_variable))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_variable(&pkg_client, &world, want_variable, 2);
 	}
 
 	if ((want_flags & PKG_REQUIRES) == PKG_REQUIRES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_requires, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
+		apply_requires(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_REQUIRES_PRIVATE) == PKG_REQUIRES_PRIVATE)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SEARCH_PRIVATE);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_requires_private, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
-
-		pkgconf_client_set_flags(&pkg_client, want_client_flags);
+		apply_requires_private(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_CFLAGS))
 	{
-		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SEARCH_PRIVATE);
-
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_cflags, maximum_traverse_depth, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out_println;
-		}
-
-		pkgconf_client_set_flags(&pkg_client, want_client_flags);
+		apply_cflags(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_LIBS))
 	{
-		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_libs, maximum_traverse_depth, NULL))
+		if (want_flags & PKG_CFLAGS)
+			printf(" ");
+
+		if (!(want_flags & PKG_STATIC))
 		{
-			ret = EXIT_FAILURE;
-			goto out_println;
+			pkgconf_client_set_flags(&pkg_client, pkg_client.flags & ~PKGCONF_PKG_PKGF_SEARCH_PRIVATE);
+
+			/* redo the solution for the library set: free the solution itself, and any cached graph nodes */
+			pkgconf_solution_free(&pkg_client, &world);
+			pkgconf_cache_free(&pkg_client);
+
+			if (!pkgconf_queue_solve(&pkg_client, &pkgq, &world, maximum_traverse_depth))
+			{
+				ret = EXIT_FAILURE;
+				goto out;
+			}
 		}
+
+		apply_libs(&pkg_client, &world, NULL, 2);
 	}
 
-	pkgconf_queue_free(&pkgq);
-
-out_println:
 	if (want_flags & (PKG_CFLAGS|PKG_LIBS))
 		printf("\n");
 
 out:
+	pkgconf_solution_free(&pkg_client, &world);
+	pkgconf_queue_free(&pkgq);
 	pkgconf_cross_personality_deinit(personality);
 	pkgconf_client_deinit(&pkg_client);
 
