@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <string.h>
+#include <errno.h>
 
 #include <libpkgconf/bsdstubs.h>
 #include <libpkgconf/config.h>
@@ -137,3 +138,23 @@ pkgconf_strndup(const char *src, size_t len)
 {
 	return strndup(src, len);
 }
+
+#ifndef HAVE_REALLOCARRAY
+void *
+reallocarray(void *ptr, size_t m, size_t n)
+{
+	if (n && m > -1 / n)
+	{
+		errno = ENOMEM;
+		return 0;
+	}
+
+	return realloc(ptr, m * n);
+}
+#endif
+
+void *
+pkgconf_reallocarray(void *ptr, size_t m, size_t n)
+{
+	return reallocarray(ptr, m, n);
+}
